#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

#include <pldstr.h>
#include <logger.h>
#include <replace-text.h>


struct REPORTGEN_object {
	char *var_filename;
	char *raw_filename;
	char *raw_report;
	char *cooked_report;
};


/**
 **
 1. Load variables file
	
 2. Load report file
 3. parse and generate output 

 **
 **/

static unsigned char hexconv[256]={
	0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    1,    2,    3,    4,    5,    6,    7,    8,    9,    0,    0,    0,    0,    0,    0,\
		0,   10,   11,   12,   13,   14,   15,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,   10,   11,   12,   13,   14,   15,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,\
		0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,    0 \
};

/*-----------------------------------------------------------------\
 Function Name	: REPORTGEN_uri2str
 Returns Type	: int
 	----Parameter List
	1. char *convme , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int REPORTGEN_uri2str( char *convme )
{
	char *o, *n; // * original, new


	o = n = convme;

	while (*o != '\0')
	{
		switch (*o)
		{
			case '+':
				*n = ' ';
				break;

			case '%':
				*n = ((char)hexconv[(int)(*(o+1))]*16 +hexconv[(int)(*(o+2))]);
				o+=2;
				break;

			default:
				*n = *o;
		}
		o++;
		n++;
	}

	*n = '\0';

	return 0;
}


/*-----------------------------------------------------------------\
 Function Name	: REPORTGEN_init
 Returns Type	: int
 	----Parameter List
	1. struct REPORTGEN_object *rg , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int REPORTGEN_init( struct REPORTGEN_object *rg )
{
	rg->var_filename = NULL;
	rg->raw_filename = NULL;
	rg->raw_report = NULL;
	rg->cooked_report = NULL;

	return 0;
}

/*-----------------------------------------------------------------\
 Function Name	: REPORTGEN_set_raw_filename
 Returns Type	: int
 	----Parameter List
	1. struct REPORTGEN_object *rg, 
	2.  char *filename , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int REPORTGEN_set_raw_filename( struct REPORTGEN_object *rg, char *filename )
{
	if (filename) rg->raw_filename = strdup(filename);

	return 0;
}


/*-----------------------------------------------------------------\
 Function Name	: REPORTGEN_set_var_filename
 Returns Type	: int
 	----Parameter List
	1. struct REPORTGEN_object *rg, 
	2.  char *filename , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int REPORTGEN_set_var_filename( struct REPORTGEN_object *rg, char *filename )
{
	if (filename) rg->var_filename = strdup(filename);

	return 0;
}



/*-----------------------------------------------------------------\
 Function Name	: REPORTGEN_load_report
 Returns Type	: int
 	----Parameter List
	1. struct REPORTGEN_object *rg , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int REPORTGEN_load_report( struct REPORTGEN_object *rg )
{
	FILE *f;
	struct stat st;
	int stat_result;

	stat_result = stat( rg->raw_filename, &st );
	if (stat_result != 0)
	{
		return  -1;
	}

	f = fopen( rg->raw_filename, "r" );
	if (f == NULL)
	{
		return -1;
	}

	rg->raw_report = malloc(sizeof(char) *(1 +st.st_size));
	if (rg->raw_report == NULL)
	{
		return -1;
	}

	if ((f != NULL)&&(rg->raw_report != NULL))
	{
		int read_count;

		read_count = fread( rg->raw_report, sizeof(char), st.st_size, f);
		if (read_count != st.st_size)
		{
			// complain that the read data size doesn't match
		}

		*(rg->raw_report +st.st_size) = '\0';

		fclose(f);
	}

	return 0;

}

/*-----------------------------------------------------------------\
 Function Name	: REPORTGEN_load_varfile
 Returns Type	: int
 	----Parameter List
	1. struct REPORTGEN_object *rg, 
	2.  struct RPLTXT_object *rpl , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int REPORTGEN_load_vars( struct REPORTGEN_object *rg, struct RPLTXT_object *rpl )
{
	char line[10000];
	FILE *f;

	f = fopen(rg->var_filename,"r");
	if (f == NULL) return -1;

	while (fgets(line, sizeof(line), f))
	{
		char *p;
		char vname[100];
		int ll;

		/** Drop off the trailing \n **/
		ll = strlen(line);
		if (line[ll -1] == '\n') line[ll -1] = '\0';

		/** Split the name=value pairing **/
		p = strchr(line,'=');
		if (p == NULL) continue;

		/** Move p to point to the start of the variable **/
		*p = '\0';
		p++; /** move past the = separator. **/

		/** Prefix the variable marker **/
		snprintf(vname,sizeof(vname),"$%s", line);

		/** Decode URI encoded stream **/
		REPORTGEN_uri2str(p);
		RPLTXT_add_variable( rpl, vname, ((strlen(p)<1)?" ":p) );
	}

	fclose(f);

	return 0;
}



/*-----------------------------------------------------------------\
 Function Name	: REPORTGEN_dump_report
 Returns Type	: int
 	----Parameter List
	1. struct REPORTGEN_object *rg, 
	2.  FILE *output , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int REPORTGEN_dump_report( struct REPORTGEN_object *rg, FILE *output )
{
	fprintf(output,"%s",rg->cooked_report);

	return 0;
}

/*-----------------------------------------------------------------\
 Function Name	: REPORTGEN_done
 Returns Type	: int
 	----Parameter List
	1. REPORTGEN_object *rg, 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int REPORTGEN_done( struct REPORTGEN_object *rg)
{
	if (rg->raw_filename) free(rg->raw_filename);
	if (rg->raw_report) free(rg->raw_report);
	if (rg->cooked_report) free(rg->cooked_report);

	return 0;
}

/*-----------------------------------------------------------------\
 Function Name	: main
 Returns Type	: int
 	----Parameter List
	1. int argc, 
	2.  char **argv , 
 	------------------
 Exit Codes	: 
 Side Effects	: 
--------------------------------------------------------------------
 Comments:
 
--------------------------------------------------------------------
 Changes:
 
\------------------------------------------------------------------*/
int main(int argc, char **argv )
{
	struct REPORTGEN_object rg;
	struct RPLTXT_object rpl;

	RPLTXT_init(&rpl);
	REPORTGEN_init(&rg);
	REPORTGEN_set_raw_filename(&rg, argv[1]);
	REPORTGEN_set_var_filename(&rg, argv[2]);
	REPORTGEN_load_report(&rg);
	REPORTGEN_load_vars(&rg, &rpl);

	rg.cooked_report = RPLTXT_convert(&rpl, rg.raw_report);
	REPORTGEN_dump_report(&rg, stdout);
	RPLTXT_done(&rpl);

	REPORTGEN_done(&rg);

	return 0;
}
